@@ -99,7 +99,7 @@ brmssummary <- function(formula = NULL, family = "", link = "",
 #'  When interpreting Bayes factors, make sure 
 #'  that your priors are reasonable and carefully chosen,
 #'  as the result will depend heavily on the priors. 
-#'  It particular, avoid using default priors.
+#'  In particular, avoid using default priors.
 #'  
 #'  The argument \code{alpha} specifies the size of the credible interval
 #'  (i.e., Bayesian confidence interval).
@@ -565,7 +565,7 @@ stanplot <- function(object, ...) {
 #' Display marginal effects of one or more numeric and/or categorical 
 #' predictors including two-way interaction effects.
 #' 
-#' @param x An object usually of class \code{brmsfit}.
+#' @param x An \R object usually of class \code{brmsfit}.
 #' @param effects An optional character vector naming effects
 #'   (main effects or interactions) for which to compute marginal plots.
 #'   Interactions are specified by a \code{:} between variable names.
@@ -579,7 +579,17 @@ stanplot <- function(object, ...) {
 #'   The row names of \code{data} will be treated as titles of the subplots. 
 #'   It is recommended to only define a few rows in order to keep the plots clear.
 #'   If \code{NULL} (the default), numeric variables will be marginalized
-#'   by using their means and factors will get their reference level assigned.   
+#'   by using their means and factors will get their reference level assigned.
+#' @param int_conditions An optional named \code{list} whose elements are numeric
+#'   vectors of values of the second variables in two-way interactions. 
+#'   At these values, predictions are evaluated. The names of 
+#'   \code{int_conditions} have to match the variable names exactly.
+#'   Additionally, the elements of the numeric vectors may be named themselves,
+#'   in which case their names appear as labels for the conditions in the plots.
+#'   Instead of vectors, functions returning vectors may be passed and are
+#'   applied on the original values of the corresponding variable.
+#'   If \code{NULL} (the default), predictions are evaluated at the 
+#'   \eqn{mean} and at \eqn{mean +/- sd}. 
 #' @param re_formula A formula containing random effects to be considered 
 #'   in the marginal predictions. If \code{NULL}, include all random effects; 
 #'   if \code{NA} (default), include no random effects.
@@ -600,12 +610,21 @@ stanplot <- function(object, ...) {
 #'   this implies \code{10000} support points for interaction terms,
 #'   so it might be necessary to reduce \code{resolution} 
 #'   when only few RAM is available.
-#' @param too_far For surface plots only: Grid points that are too 
+#' @param too_far Positive number. 
+#'   For surface plots only: Grid points that are too 
 #'   far away from the actual data points can be excluded from the plot. 
 #'   \code{too_far} determines what is too far. The grid is scaled into 
 #'   the unit square and then grid points more than \code{too_far} 
 #'   from the predictor variables are excluded. By default, all
 #'   grid points are used. Ignored for non-surface plots.
+#' @param select_points Positive number. 
+#'   Only relevant if \code{points} or \code{rug} are set to \code{TRUE}: 
+#'   Actual data points of numeric variables that 
+#'   are too far away from the values specified in \code{conditions} 
+#'   can be excluded from the plot. Values are scaled into 
+#'   the unit interval and then points more than \code{select_points} 
+#'   from the values in \code{conditions} are excluded. 
+#'   By default, all points are used.
 #' @param ncol Number of plots to display per column for each effect.
 #'   If \code{NULL} (default), \code{ncol} is computed internally based
 #'   on the number of rows of \code{data}.
@@ -613,9 +632,17 @@ stanplot <- function(object, ...) {
 #'   should be added via \code{\link[ggplot2:geom_point]{geom_point}}.
 #'   Default is \code{FALSE}. Note that only those data points will be added
 #'   that match the specified conditions defined in \code{conditions}.
+#'   For categorical predictors, the conditions have to match exactly. 
+#'   For numeric predictors, argument \code{select_points} is used to
+#'   determine, which points do match a condition.
 #' @param rug Logical; indicating whether a rug representation of predictor
 #'   values should be added via \code{\link[ggplot2:geom_rug]{geom_rug}}.
-#'   Default is \code{FALSE}.
+#'   Default is \code{FALSE}. Depends on \code{select_points} in the same
+#'   way as \code{points} does.
+#' @param jitter_width Only used if \code{points = TRUE}: 
+#'   Amount of horizontal jittering of the data points.
+#'   Mainly useful for ordinal models. Defaults to \code{0} that 
+#'   is no jittering.
 #' @param stype Indicates how surface plots should be displayed.
 #'   Either \code{"contour"} or \code{"raster"}.
 #' @inheritParams plot.brmsfit
@@ -655,6 +682,12 @@ stanplot <- function(object, ...) {
 #'   are interpreted as if all dummy variables of this factor are 
 #'   zero. This allows, for instance, to make predictions of the grand mean 
 #'   when using sum coding. 
+#'   
+#'   To fully change colours of the created plots, 
+#'   one has to amend both \code{scale_colour} and \code{scale_fill}.
+#'   See \code{\link[ggplot2:scale_colour_grey]{scale_colour_grey}} or
+#'   \code{\link[ggplot2:scale_colour_gradient]{scale_colour_gradient}}
+#'   for more details.
 #' 
 #' @examples 
 #' \dontrun{
@@ -664,6 +697,12 @@ stanplot <- function(object, ...) {
 #' ## plot all marginal effects
 #' plot(marginal_effects(fit), ask = FALSE)
 #' 
+#' ## change colours to grey scale
+#' me <- marginal_effects(fit, "log_Base4_c:Trt_c")
+#' plot(me, plot = FALSE)[[1]] + 
+#'   scale_color_grey() +
+#'   scale_fill_grey()
+#' 
 #' ## only plot the marginal interaction effect of 'log_Base4_c:Trt_c'
 #' ## for different values for 'log_Age_c'
 #' conditions <- data.frame(log_Age_c = c(-0.3, 0, 0.3))
@@ -675,13 +714,29 @@ stanplot <- function(object, ...) {
 #' plot(marginal_effects(fit, effects = "log_Base4_c:Trt_c", 
 #'                       conditions = conditions, re_formula = NULL), 
 #'      points = TRUE, rug = TRUE)
+#'  
+#' ## change handling of two-way interactions
+#' int_conditions <- list(
+#'   log_Base4_c = setNames(c(-2, 1, 0), c("b", "c", "a"))
+#' )
+#' marginal_effects(fit, effects = "Trt_c:log_Base4_c",
+#'                  int_conditions = int_conditions)
+#' marginal_effects(fit, effects = "Trt_c:log_Base4_c",
+#'                  int_conditions = list(log_Base4_c = quantile))        
 #'      
 #' ## fit a model to illustrate how to plot 3-way interactions
 #' fit3way <- brm(count ~ log_Age_c * log_Base4_c * Trt_c, data = epilepsy)
 #' conditions <- data.frame(log_Age_c = c(-0.3, 0, 0.3))
 #' rownames(conditions) <- paste("log_Age_c =", conditions$log_Age_c)
-#' plot(marginal_effects(fit3way, "log_Base4_c:Trt_c",
-#'                       conditions = conditions))
+#' marginal_effects(
+#'   fit3way, "log_Base4_c:Trt_c", conditions = conditions
+#' )
+#' ## only include points close to the specified values of log_Age_c
+#' me <- marginal_effects(
+#'  fit3way, "log_Base4_c:Trt_c", conditions = conditions, 
+#'  select_points = 0.1
+#' )
+#' plot(me, points = TRUE)
 #' }
 #' 
 #' @export
@@ -737,6 +792,73 @@ marginal_smooths <- function(x, ...) {
   UseMethod("marginal_smooths")
 }
 
+#' Posterior Probabilities of Mixture Component Memberships
+#' 
+#' Compute the posterior probabilities of mixture component 
+#' memberships for each observation including uncertainty
+#' estimates.
+#' 
+#' @inheritParams predict.brmsfit
+#' @param x An \R object usually of class \code{brmsfit}.
+#' @param log Logical; Indicates whether to return 
+#'   probabilities on the log-scale.
+#' 
+#' @return 
+#' If \code{summary = TRUE}, an N x E x K array,
+#' where N is the number of observations, K is the number
+#' of mixture components, and E is equal to \code{length(probs) + 2}.
+#' If \code{summary = FALSE}, an S x N x K arrary, where
+#' S is the number of posterior samples.
+#' 
+#' @details 
+#' The returned probabilities can be written as
+#' \eqn{P(Kn = k | Yn)}, that is the posterior probability 
+#' that observation n orginiates from component k. 
+#' They are computed using Bayes' Theorem
+#' \deqn{P(Kn = k | Yn) = P(Yn | Kn = k) P(Kn = k) / P(Yn),}
+#' where \eqn{P(Yn | Kn = k)} is the (posterior) likelihood
+#' of observation n for component k, \eqn{P(Kn = k)} is 
+#' the (posterior) mixing probability of component k 
+#' (i.e. parameter \code{theta<k>}), and 
+#' \deqn{P(Yn) = \sum (k=1,...,K) P(Yn | Kn = k) P(Kn = k)}
+#' is a normalizing constant.
+#' 
+#' @examples 
+#' \dontrun{
+#' ## simulate some data
+#' set.seed(1234)
+#' dat <- data.frame(
+#'   y = c(rnorm(100), rnorm(50, 2)), 
+#'   x = rnorm(150)
+#' )
+#' ## fit a simple normal mixture model
+#' mix <- mixture(gaussian, nmix = 2)
+#' prior <- c(
+#'   prior(normal(0, 5), Intercept, nlpar = mu1),
+#'   prior(normal(0, 5), Intercept, nlpar = mu2),
+#'   prior(dirichlet(2, 2), theta)
+#' )
+#' fit1 <- brm(bf(y ~ x), dat, family = mix,
+#'             prior = prior, chains = 2, inits = 0)
+#' summary(fit1)
+#'    
+#' ## compute the membership probabilities         
+#' ppm <- pp_mixture(fit1)
+#' str(ppm)
+#' 
+#' ## extract point estimates for each observation
+#' head(ppm[, 1, ])
+#' 
+#' ## classify every observation according to 
+#' ## the most likely component
+#' apply(ppm[, 1, ], 1, which.max)
+#' }
+#' 
+#' @export
+pp_mixture <- function(x, ...) {
+  UseMethod("pp_mixture")
+}
+
 #' Expose user-defined \pkg{Stan} functions
 #' 
 #' Export user-defined \pkg{Stan} function to the 
@@ -752,6 +874,24 @@ expose_functions <- function(x, ...) {
   UseMethod("expose_functions")
 }
 
+#' Extract Control Parameters of the NUTS Sampler
+#' 
+#' Extract control parameters of the NUTS sampler such as 
+#' \code{adapt_delta} or \code{max_treedepth}.
+#' 
+#' @param x an \R object
+#' @param pars Optional names of the control parameters to be returned.
+#'  If \code{NULL} (the default) all control parameters are returned.
+#'  See \code{\link[rstan:stan]{stan}} for more details.
+#' @param ... Currently ignored.
+#' 
+#' @return A named \code{list} with control parameter values.
+#' 
+#' @export
+control_params <- function(x, ...) {
+  UseMethod("control_params")
+}
+
 #' Extract Diagnostic Quantities of \pkg{brms} Models
 #' 
 #' Extract quantities that can be used to diagnose sampling behavior
@@ -835,3 +975,22 @@ check_prior_special <- function(x, ...) {
   # prepare special priors for use in Stan
   UseMethod("check_prior_special")
 }
+
+auxpar_family <- function(family, auxpar, ...) {
+  # generate a family object of an auxiliary parameter
+  UseMethod("auxpar_family")
+}
+
+family_names <- function(family, ...) {
+  # extract family names
+  UseMethod("family_names")
+}
+
+valid_auxpars <- function(family, ...) {
+  # get valid auxiliary parameters for a family
+  UseMethod("valid_auxpars")
+}
+
+stan_llh <- function(family, ...) {
+  UseMethod("stan_llh")
+}
