@@ -20,14 +20,14 @@ predict_student <- function(i, draws, ...) {
   args <- list(df = get_auxpar(draws$nu, i = i), 
                mu = ilink(get_eta(draws$mu, i), draws$f$link), 
                sigma = get_sigma(draws$sigma, data = draws$data, i = i))
-  rng_continuous(nrng = draws$nsamples, dist = "student", args = args, 
+  rng_continuous(nrng = draws$nsamples, dist = "student_t", args = args, 
                  lb = draws$data$lb[i], ub = draws$data$ub[i])
 }
 
 predict_cauchy <- function(i, draws, ...) {
   args <- list(df = 1, mu = ilink(get_eta(draws$mu, i), draws$f$link), 
                sigma = get_sigma(draws$sigma, data = draws$data, i = i))
-  rng_continuous(nrng = draws$nsamples, dist = "student", args = args,
+  rng_continuous(nrng = draws$nsamples, dist = "student_t", args = args,
                  lb = draws$data$lb[i], ub = draws$data$ub[i])
 }
 
@@ -62,8 +62,8 @@ predict_student_mv <- function(i, draws, ...) {
   }
   nu <- get_auxpar(draws$nu, i = i)
   .fun <- function(s) {
-    rmulti_student(1, df = nu[s, ], mu = mu[s, ], 
-                   Sigma = draws$Sigma[s, , ])
+    rmulti_student_t(1, df = nu[s, ], mu = mu[s, ], 
+                     Sigma = draws$Sigma[s, , ])
   }
   do.call(rbind, lapply(1:draws$nsamples, .fun))
 }
@@ -77,8 +77,8 @@ predict_cauchy_mv <- function(i, draws, ...) {
     mu <- ilink(get_eta(draws$mu, i), draws$f$link)[, 1, ]
   }
   .fun <- function(s) {
-    rmulti_student(1, df = 1, mu = mu[s, ],
-                   Sigma = draws$Sigma[s, , ])
+    rmulti_student_t(1, df = 1, mu = mu[s, ],
+                     Sigma = draws$Sigma[s, , ])
   }
   do.call(rbind, lapply(1:draws$nsamples, .fun))
 }
@@ -135,8 +135,8 @@ predict_student_cov <- function(i, draws, ...) {
   }
   nu <- get_auxpar(draws$nu, i = i)
   .fun <- function(s) {
-    rmulti_student(1, df = nu[s, ], mu = mu[s, ], 
-                   Sigma = Sigma[s, , ])
+    rmulti_student_t(1, df = nu[s, ], mu = mu[s, ], 
+                     Sigma = Sigma[s, , ])
   }
   do.call(rbind, lapply(1:draws$nsamples, .fun))
 }
@@ -160,7 +160,7 @@ predict_student_fixed <- function(i, draws, ...) {
   mu <- ilink(get_eta(draws$mu, 1:nrow(draws$data$V)), draws$f$link)
   nu <- get_auxpar(draws$nu, 1:nrow(draws$data$V))
   .fun <- function(s) {
-    rmulti_student(1, df = nu[s, ], mu = mu[s, ], Sigma = draws$data$V)
+    rmulti_student_t(1, df = nu[s, ], mu = mu[s, ], Sigma = draws$data$V)
   }
   do.call(rbind, lapply(1:draws$nsamples, .fun))
 }
@@ -261,14 +261,23 @@ predict_exgaussian <- function(i, draws, ...) {
                  lb = draws$data$lb[i], ub = draws$data$ub[i])
 }
 
-predict_wiener <- function(i, draws, col = c("q", "resp"), ...) {
-  args <- list(delta = ilink(get_eta(draws$mu, i), draws$f$link), 
-               alpha = get_auxpar(draws$bs, i = i),
-               tau = get_auxpar(draws$ndt, i = i),
-               beta = get_auxpar(draws$bias, i = i),
-               col = match.arg(col))
-  rng_continuous(nrng = 1, dist = "Wiener", args = args, 
-                 lb = draws$data$lb[i], ub = draws$data$ub[i])
+predict_wiener <- function(i, draws, negative_rt = FALSE, ...) {
+  args <- list(
+    delta = ilink(get_eta(draws$mu, i), draws$f$link), 
+    alpha = get_auxpar(draws$bs, i = i),
+    tau = get_auxpar(draws$ndt, i = i),
+    beta = get_auxpar(draws$bias, i = i),
+    types = if (negative_rt) c("q", "resp") else "q"
+  )
+  out <- rng_continuous(
+    nrng = 1, dist = "wiener", args = args, 
+    lb = draws$data$lb[i], ub = draws$data$ub[i]
+  )
+  if (negative_rt) {
+    # code lower bound responses as negative RTs
+    out <- out[["q"]] * ifelse(out[["resp"]], 1, -1)
+  }
+  out
 }
 
 predict_beta <- function(i, draws, ...) {
@@ -296,7 +305,7 @@ predict_asym_laplace <- function(i, draws, ...) {
 
 predict_hurdle_poisson <- function(i, draws, ...) {
   # theta is the bernoulli hurdle parameter
-  theta <- get_theta(draws, i, par = "hu")
+  theta <- get_zi_hu(draws, i, par = "hu")
   lambda <- ilink(get_eta(draws$mu, i), draws$f$link)
   ndraws <- draws$nsamples
   # compare with theta to incorporate the hurdle process
@@ -309,7 +318,7 @@ predict_hurdle_poisson <- function(i, draws, ...) {
 
 predict_hurdle_negbinomial <- function(i, draws, ...) {
   # theta is the bernoulli hurdle parameter
-  theta <- get_theta(draws, i, par = "hu")
+  theta <- get_zi_hu(draws, i, par = "hu")
   mu <- ilink(get_eta(draws$mu, i), draws$f$link)
   ndraws <- draws$nsamples
   # compare with theta to incorporate the hurdle process
@@ -323,7 +332,7 @@ predict_hurdle_negbinomial <- function(i, draws, ...) {
 
 predict_hurdle_gamma <- function(i, draws, ...) {
   # theta is the bernoulli hurdle parameter
-  theta <- get_theta(draws, i, par = "hu")
+  theta <- get_zi_hu(draws, i, par = "hu")
   shape <- get_shape(draws$shape, data = draws$data, i = i)
   scale <- ilink(get_eta(draws$mu, i), draws$f$link) / shape
   ndraws <- draws$nsamples
@@ -334,7 +343,7 @@ predict_hurdle_gamma <- function(i, draws, ...) {
 
 predict_hurdle_lognormal <- function(i, draws, ...) {
   # theta is the bernoulli hurdle parameter
-  theta <- get_theta(draws, i, par = "hu")
+  theta <- get_zi_hu(draws, i, par = "hu")
   mu <- ilink(get_eta(draws$mu, i), draws$f$link)
   sigma <- get_sigma(draws$sigma, data = draws$data, i = i)
   ndraws <- draws$nsamples
@@ -345,7 +354,7 @@ predict_hurdle_lognormal <- function(i, draws, ...) {
 
 predict_zero_inflated_beta <- function(i, draws, ...) {
   # theta is the bernoulli hurdle parameter
-  theta <- get_theta(draws, i, par = "zi")
+  theta <- get_zi_hu(draws, i, par = "zi")
   mu <- ilink(get_eta(draws$mu, i), draws$f$link)
   phi <- get_auxpar(draws$phi, i = i)
   # compare with theta to incorporate the hurdle process
@@ -356,7 +365,7 @@ predict_zero_inflated_beta <- function(i, draws, ...) {
 
 predict_zero_inflated_poisson <- function(i, draws, ...) {
   # theta is the bernoulli zero-inflation parameter
-  theta <- get_theta(draws, i, par = "zi")
+  theta <- get_zi_hu(draws, i, par = "zi")
   lambda <- ilink(get_eta(draws$mu, i), draws$f$link)
   ndraws <- draws$nsamples
   # compare with theta to incorporate the zero-inflation process
@@ -366,7 +375,7 @@ predict_zero_inflated_poisson <- function(i, draws, ...) {
 
 predict_zero_inflated_negbinomial <- function(i, draws, ...) {
   # theta is the bernoulli zero-inflation parameter
-  theta <- get_theta(draws, i, par = "zi")
+  theta <- get_zi_hu(draws, i, par = "zi")
   mu <- ilink(get_eta(draws$mu, i), draws$f$link)
   shape <- get_shape(draws$shape, data = draws$data, i = i)
   ndraws <- draws$nsamples
@@ -377,7 +386,7 @@ predict_zero_inflated_negbinomial <- function(i, draws, ...) {
 
 predict_zero_inflated_binomial <- function(i, draws, ...) {
   # theta is the bernoulii zero-inflation parameter
-  theta <- get_theta(draws, i, par = "zi")
+  theta <- get_zi_hu(draws, i, par = "zi")
   trials <- draws$data$trials[i]
   prob <- ilink(get_eta(draws$mu, i), draws$f$link)
   ndraws <- draws$nsamples
@@ -418,6 +427,31 @@ predict_ordinal <- function(i, draws, family, ...) {
   first_greater(p, target = runif(draws$nsamples, min = 0, max = 1))
 }
 
+predict_mixture <- function(i, draws, ...) {
+  families <- family_names(draws$f)
+  theta <- get_theta(draws, i = i)
+  smix <- rng_mix(theta)
+  out <- rep(NA, draws$nsamples)
+  for (j in seq_along(families)) {
+    sample_ids <- which(smix == j)
+    if (length(sample_ids)) {
+      predict_fun <- paste0("predict_", families[j])
+      predict_fun <- get(predict_fun, asNamespace("brms"))
+      auxpars <- valid_auxpars(families[j])
+      tmp_draws <- list(
+        f = draws$f$mix[[j]],
+        nsamples = length(sample_ids),
+        data = draws[["data"]]
+      )
+      for (ap in auxpars) {
+        tmp_draws[[ap]] <- p(draws[[paste0(ap, j)]], sample_ids, row = TRUE)
+      }
+      out[sample_ids] <- predict_fun(i, tmp_draws, ...)
+    }
+  }
+  out
+}
+
 #---------------predict helper-functions----------------------------
 
 rng_continuous <- function(nrng, dist, args, lb = NULL, ub = NULL) {
@@ -475,6 +509,16 @@ rng_discrete <- function(nrng, dist, args, lb = NULL, ub = NULL, ntrys = 5) {
   }
 }
 
+rng_mix <- function(theta) {
+  # sample the ID of the mixture component
+  #stopifnot(all(dim(theta), c(nrng, length(families))))
+  stopifnot(is.matrix(theta))
+  mix_comp <- seq_len(ncol(theta))
+  ulapply(seq_len(nrow(theta)), function(s)
+    sample(mix_comp, 1, prob = theta[s, ])
+  )
+}
+
 extract_valid_sample <- function(rng, lb, ub) {
   # extract the first valid predicted value 
   # per Stan sample per observation 
