@@ -379,8 +379,8 @@ set_prior <- function(prior, class = "b", coef = "", group = "",
     
   valid_classes <- c("Intercept", "b", "sd", "sds", "simplex", "cor", "L", 
                      "ar", "ma", "arr", "sigmaLL", "rescor", "Lrescor", 
-                     "delta", auxpars(), if (!check) "")
-  if (!class %in% valid_classes) {
+                     "delta", "theta", if (!check) "")
+  if (!(class %in% valid_classes || auxpar_class(class) %in% auxpars())) {
     stop2("'", class, "' is not a valid parameter class.")
   }
   if (nchar(group) && !class %in% c("sd", "cor", "L")) {
@@ -523,7 +523,9 @@ get_prior <- function(formula, data, family = NULL,
   # ensure that RE and residual SDs only have a weakly informative prior by default
   Y <- unname(model.response(data))
   prior_scale <- 10
-  if (is_lognormal(family)) link <- "log"
+  if (is_lognormal(family)) {
+    link <- "log"
+  }
   if (link %in% c("identity", "log", "inverse", "sqrt", "1/mu^2")) {
     if (link %in% c("log", "inverse", "1/mu^2")) {
       Y <- ifelse(Y == 0, Y + 0.1, Y)  # avoid Inf in link(Y)
@@ -569,13 +571,14 @@ get_prior <- function(formula, data, family = NULL,
   )
   valid_auxpars <- valid_auxpars(family, bterms = bterms)
   for (ap in valid_auxpars) {
+    ap_class <- auxpar_class(ap)
     if (!is.null(bterms$auxpars[[ap]])) {
       auxprior <- prior_effects(
         bterms$auxpars[[ap]], data = data, nlpar = ap,
         def_scale_prior = def_scale_prior
       )
-    } else if (!is.na(def_auxprior[ap])) {
-      auxprior <- brmsprior(class = ap, prior = def_auxprior[ap])
+    } else if (!is.na(def_auxprior[ap_class])) {
+      auxprior <- brmsprior(class = ap, prior = def_auxprior[ap_class])
     } else {
       auxprior <- empty_brmsprior()
     }
@@ -592,6 +595,11 @@ get_prior <- function(formula, data, family = NULL,
   if (is_ordinal(family) && threshold == "equidistant") {
     prior <- rbind(prior, brmsprior(class = "delta"))
   }
+  # priors for mixture models
+  ap_classes <- auxpar_class(names(c(bterms$auxpars, bterms$fauxpars)))
+  if (is.mixfamily(family) && !any(ap_classes == "theta")) {
+    prior <- rbind(prior, brmsprior(class = "theta"))
+  }
   # priors for auxiliary parameters of multivariate models
   if (is_linear(family) && length(bterms$response) > 1L) {
     sigma_coef <- c("", bterms$response)
@@ -927,16 +935,28 @@ check_prior <- function(prior, formula, data = NULL, family = NULL,
   for (k in seq_along(mo_forms)) {
     monef <- colnames(get_model_matrix(mo_forms[[k]], data = data))
     for (i in seq_along(monef)) {
-      take <- with(prior, 
+      take <- with(prior,
         class == "simplex" & coef == monef[i] & nlpar == names(mo_forms)[k]
       )
-      simplex_prior <- paste0(".", prior$prior[take])
-      if (nchar(simplex_prior) > 1L) {
-        simplex_prior <- paste(eval2(simplex_prior), collapse = ",")
+      simplex_prior <- prior$prior[take]
+      if (isTRUE(nzchar(simplex_prior))) {
+        # hard code prior concentration 
+        # in order not to depend on external objects
+        simplex_prior <- paste0(eval2(simplex_prior), collapse = ", ")
         prior$prior[take] <- paste0("dirichlet(c(", simplex_prior, "))")
       }
     }
   }
+  # prepare priors for mixture probabilities
+  if (is.mixfamily(family)) {
+    take <- prior$class == "theta"
+    theta_prior <- prior$prior[take]
+    if (isTRUE(nzchar(theta_prior))) {
+      # hard code prior concentration
+      theta_prior <- paste0(eval2(theta_prior), collapse = ", ")
+      prior$prior[take] <- paste0("dirichlet(c(", theta_prior, "))")
+    }
+  }
   if (length(rows2remove)) {   
     prior <- prior[-rows2remove, ]
   }
@@ -1009,7 +1029,7 @@ check_prior_content <- function(prior, family = gaussian(), warn = TRUE) {
         if (prior$bound[i] != "<lower=-1,upper=1>") {
           autocor_warning <- TRUE
         } 
-      } else if (prior$class[i] == "simplex") {
+      } else if (prior$class[i] %in% c("simplex", "theta")) {
         if (nchar(prior$prior[i]) && !grepl("^dirichlet\\(", prior$prior[i])) {
           stop2("Currently 'dirichlet' is the only valid prior ",
                 "for simplex parameters. See help(set_prior) ",
